@@ -1,8 +1,8 @@
 """
 Units models
 """
-
 import logging
+import re
 from datetime import date
 
 import pint.systems
@@ -11,17 +11,19 @@ from django.contrib.auth.models import User
 from django.db import models
 from django.utils.translation import ugettext as _
 
-from djangophysics.countries.models import Country
 from djangophysics.converters.models import BaseConverter, ConverterResult, \
     ConverterResultDetail, ConverterResultError, ConverterLoadError
+from djangophysics.countries.models import Country
 from . import UNIT_EXTENDED_DEFINITION, DIMENSIONS, \
     UNIT_SYSTEM_BASE_AND_DERIVED_UNITS, \
     ADDITIONAL_BASE_UNITS, PREFIX_SYMBOL
 from .exceptions import UnitConverterInitError, DimensionNotFound, \
     UnitSystemNotFound, UnitNotFound, \
     UnitDuplicateError, UnitDimensionError, \
-    UnitValueError
-from .settings import ADDITIONAL_UNITS, PREFIXED_UNITS_DISPLAY
+    UnitValueError, DimensionValueError, DimensionDimensionError, \
+    DimensionDuplicateError
+from .settings import ADDITIONAL_UNITS, ADDITIONAL_DIMENSIONS, \
+    PREFIXED_UNITS_DISPLAY
 
 
 class Quantity:
@@ -65,6 +67,11 @@ class UnitSystem:
     system_name = None
     system = None
     _additional_units = set()
+    dimensions_cache = {}
+    units_cache = {}
+    user = None
+    key = None
+    _additional_dimensions = set()
 
     def __init__(self, system_name: str = 'SI',
                  fmt_locale: str = 'en', user: User = None,
@@ -81,22 +88,31 @@ class UnitSystem:
         if not found:
             raise UnitSystemNotFound("Invalid unit system")
         self.system_name = system_name
+        try:
+            additional_dimensions_settings = settings.PHYSICS_ADDITIONAL_DIMENSIONS
+        except AttributeError:
+            additional_dimensions_settings = ADDITIONAL_DIMENSIONS
         try:
             additional_units_settings = settings.PHYSICS_ADDITIONAL_UNITS
         except AttributeError:
             additional_units_settings = ADDITIONAL_UNITS
+        self.user = user
+        self.key = key
         try:
             self.ureg = pint.UnitRegistry(
                 system=system_name,
                 fmt_locale=fmt_locale)
             self.system = getattr(self.ureg.sys, system_name)
+            self._load_additional_dimensions(
+                dimensions=additional_dimensions_settings)
             self._load_additional_units(units=ADDITIONAL_BASE_UNITS)
             self._load_additional_units(units=additional_units_settings)
             if user:
+                self._load_custom_dimensions(user=user, key=key)
                 self._load_custom_units(user=user, key=key)
             self._rebuild_cache()
-        except (FileNotFoundError, AttributeError):
-            raise UnitSystemNotFound("Invalid unit system")
+        except (FileNotFoundError, AttributeError) as e:
+            raise UnitSystemNotFound(f"Invalid unit system: {str(e)}")
 
     def _rebuild_cache(self):
         """
@@ -104,6 +120,32 @@ class UnitSystem:
         It should be in the define method of the registry
         """
         self.ureg._build_cache()
+        self.dimensions_cache = self.available_dimensions()
+        self.units_cache = self.available_units()
+
+    def _load_additional_dimensions(
+            self, dimensions: dict,
+            redefine: bool = False) -> bool:
+        """
+        Load additional base units in registry
+        """
+        available_dimensions = self.available_dimension_names()
+        if self.system_name not in dimensions:
+            logging.warning(f"error loading additional dimensions "
+                            f"for {self.system_name}")
+            return False
+        added_dimensions = []
+        for key, items in dimensions[self.system_name].items():
+            if key not in available_dimensions:
+                self.ureg.define(
+                    f"{key} = {items['relation']}")
+                added_dimensions.append(key)
+            elif redefine:
+                self.ureg.redefine(
+                    f"{key} = {items['relation']}")
+        self._additional_dimensions = self._additional_dimensions | \
+                                      set(added_dimensions)
+        return True
 
     def _load_additional_units(
             self, units: dict,
@@ -128,6 +170,43 @@ class UnitSystem:
         self._additional_units = self._additional_units | set(added_units)
         return True
 
+    def _load_custom_dimensions(
+            self,
+            user: User,
+            key: str = None,
+            redefine: bool = False) -> bool:
+        """
+        Load custom units in registry
+        """
+        if user and type(user) == User and \
+                getattr(user, 'is_authenticated', None):
+            if user.is_superuser:
+                qs = CustomDimension.objects.all()
+            else:
+                qs = CustomDimension.objects.filter(user=user)
+            if key:
+                qs = qs.filter(models.Q(key=key) | models.Q(key__isnull=True))
+        else:
+            qs = CustomDimension.objects.filter(pk=-1)
+        qs = qs.filter(
+            unit_system=self.system_name
+        ).select_related().values('code', 'relation')
+        available_dimensions = self.available_dimension_names()
+        added_dimensions = []
+        for cd in qs:
+            props = [cd['code'], cd['relation']]
+            definition = " = ".join(props)
+            if cd not in available_dimensions:
+                self.ureg.define(definition)
+                added_dimensions.append(cd['code'])
+            elif redefine:
+                self.ureg.redefine(definition)
+            else:
+                logging.error(f"{cd['code']} already defined in registry")
+        self._additional_dimensions = self._additional_dimensions | \
+                                      set(added_dimensions)
+        return True
+
     def _load_custom_units(
             self,
             user: User,
@@ -143,26 +222,28 @@ class UnitSystem:
             else:
                 qs = CustomUnit.objects.filter(user=user)
             if key:
-                qs = qs.filter(key=key)
+                qs = qs.filter(models.Q(key=key) | models.Q(key__isnull=True))
         else:
             qs = CustomUnit.objects.filter(pk=-1)
-        qs = qs.filter(unit_system=self.system_name)
+        qs = qs.filter(unit_system=self.system_name).values(
+            'code', 'relation', 'symbol', 'alias'
+        )
         available_units = self.available_unit_names()
         added_units = []
         for cu in qs:
-            props = [cu.code, cu.relation]
-            if cu.symbol:
-                props.append(cu.symbol)
-            if cu.alias:
-                props.append(cu.alias)
+            props = [cu['code'], cu['relation']]
+            if cu['symbol']:
+                props.append(cu['symbol'])
+            if cu['alias']:
+                props.append(cu['alias'])
             definition = " = ".join(props)
-            if cu.code not in available_units:
+            if cu['code'] not in available_units:
                 self.ureg.define(definition)
-                added_units.append(cu.code)
+                added_units.append(cu['code'])
             elif redefine:
                 self.ureg.redefine(definition)
             else:
-                logging.error(f"{cu.code} already defined in registry")
+                logging.error(f"{cu['code']} already defined in registry")
         self._additional_units = self._additional_units | set(added_units)
         return True
 
@@ -175,11 +256,11 @@ class UnitSystem:
         for key in units[self.system_name].keys():
             try:
                 self.unit(key).dimensionality and True
-            except pint.errors.UndefinedUnitError:
+            except (pint.errors.UndefinedUnitError, AttributeError):
                 return False
         return True
 
-    def add_definition(self, code, relation, symbol, alias):
+    def add_unit(self, code, relation, symbol, alias):
         """
         Add a new unit definition to a UnitSystem, and rebuild cache
         :param code: code of the unit
@@ -190,6 +271,17 @@ class UnitSystem:
         self.ureg.define(f"{code} = {relation} = {symbol} = {alias}")
         self._rebuild_cache()
 
+    def add_dimension(self, code, relation):
+        """
+        Add a new dimension definition to a UnitSystem, and rebuild cache
+        :param code: code of the unit
+        :param relation: relation to other units (e.g.: 3 kg/m)
+        :param symbol: short unit representation
+        :param alias: other name for unit
+        """
+        self.ureg.define(f"{code} = {relation}")
+        self._rebuild_cache()
+
     @classmethod
     def available_systems(cls) -> [str]:
         """
@@ -219,7 +311,21 @@ class UnitSystem:
         Create a Object in the UnitSystem
         :param unit_name: name of the unit in the unit system
         """
-        return Unit(unit_system=self, code=unit_name)
+        try:
+            return Unit(unit_system=self, code=unit_name)
+        except UnitNotFound:
+            return None
+
+    def available_dimension_names(self, search_term: str = None) -> [str]:
+        """
+        List of available units for a given Unit system
+        :return: Array of names of Unit systems
+        """
+        if search_term:
+            return [name for name in self.ureg._dimensions.keys()
+                    if search_term in name]
+        else:
+            return list(self.ureg._dimensions.keys())
 
     def available_unit_names(self) -> [str]:
         """
@@ -253,27 +359,44 @@ class UnitSystem:
             self,
             *arg,
             search_term: str = None,
-            ordering: str = 'name') -> {}:
+    ) -> {}:
         """
         Return available dimensions for the UnitSystem
         :param search_term: filter dimensions on name
-        :param ordering: sort result by attribute
         """
-        descending = False
-        if ordering and ordering[0] == '-':
-            ordering = ordering[1:]
-            descending = True
-        if ordering not in ['code', 'name', 'dimension']:
-            ordering = 'name'
-        if search_term:
-            dims = [Dimension(unit_system=self, code=dim)
-                       for dim in DIMENSIONS.keys() if search_term in dim]
+        dims = {}
+        if self.dimensions_cache:
+            dims = self.dimensions_cache
         else:
-            dims = [Dimension(unit_system=self, code=dim)
-                    for dim in DIMENSIONS.keys()]
-        return sorted(dims,
-                      key=lambda x: getattr(x, ordering, ''),
-                      reverse=descending)
+            for dim in self.available_dimension_names(search_term=search_term):
+                try:
+                    d = Dimension(
+                        unit_system=self,
+                        code=dim)
+                    if d:
+                        dims[dim] = d
+                except DimensionNotFound as e:
+                    logging.warning(f"dimension {dim} not found "
+                                    f"in unit system {self.system_name}")
+                    pass
+        if dims:
+            return dims
+        return dims
+
+    def available_units(self):
+        """
+        List available units
+        """
+        units = {}
+        if self.units_cache:
+            return self.units_cache
+        for unit_name in self.available_unit_names():
+            try:
+                u = self.unit(unit_name)
+                units[unit_name] = u
+            except UnitNotFound:
+                pass
+        return units
 
     @property
     def _ureg_dimensions(self):
@@ -301,35 +424,24 @@ class UnitSystem:
         except KeyError:
             return {}
 
-    def _generate_dimension_delta_dictionnary(self) -> {}:
-        """
-        Generate the dict to put in DIMENSIONS
-        """
-        output = {}
-        for dim in self._ureg_dimensions:
-            if dim not in DIMENSIONS:
-                output[dim] = {
-                    'name': f'_({dim})',
-                    'dimension': str(self._get_dimension_dimensionality(dim)),
-                    'symbol': ''
-                }
-        return output
-
-    def units_per_dimension(self, dimensions: [str]) -> {}:
+    def units_per_dimension(self, dimensions: [str] = None) -> {}:
         """
         Return units grouped by dimension
         :param dimensions: restrict list of dimensions
         """
         output = {}
-        registry_dimensions = dimensions or DIMENSIONS.keys()
-        for dim in registry_dimensions:
-            Dimension(unit_system=self, code=dim)
+        registry_dimensions = dimensions or self.available_dimension_names()
+        for uname in self.available_unit_names():
             try:
-                units = self.ureg.get_compatible_units(dim)
-                if units:
-                    output[dim] = units
-            except KeyError:
-                continue
+                u = self.unit(uname)
+                for d in [dim for dim in u.dimension_codes
+                          if dim in registry_dimensions]:
+                    if d in output:
+                        output[d].append(u)
+                    else:
+                        output[d] = [u, ]
+            except pint.errors.UndefinedUnitError:
+                pass
         return output
 
     def units_per_dimensionality(self) -> {}:
@@ -341,10 +453,10 @@ class UnitSystem:
         output = {}
         for unit_str in units_array:
             dimension = Unit.dimensionality_string(self, unit_str)
-            try:
+            if dimension in output:
                 output[dimension].append(unit_str)
-            except KeyError:
-                output[dimension] = [unit_str]
+            else:
+                output[dimension] = [unit_str, ]
         return output
 
     @property
@@ -363,24 +475,54 @@ class Dimension:
     """
     unit_system = None
     code = None
-    name = None
+    _name = None
     dimension = None
 
-    def __init__(self, unit_system: UnitSystem, code: str):
+    def __init__(self,
+                 unit_system: UnitSystem,
+                 code: str):
         """
         Initialize a Dimension in a UnitSystem
         """
-        try:
-            dimension = DIMENSIONS[code]
-            self.unit_system = unit_system
-            self.code = code
-            self.name = dimension['name']
-            self.dimension = dimension['dimension']
-        except (ValueError, KeyError) as e:
-            logging.warning(str(e))
-            self.code = None
-        if not self.code:
-            raise DimensionNotFound
+        self.unit_system = unit_system
+        self.code = code
+        if code not in ['[compounded]', '[custom]'] and \
+                self.unit_system.dimensions_cache and \
+                code not in self.unit_system.dimensions_cache.keys():
+            raise DimensionNotFound(f"Dimension {code} not found")
+
+    @property
+    def name(self):
+        """
+        Name of the dimension.
+        """
+        if self._name:
+            return self._name
+        else:
+            code = self.code
+            name = code.replace(
+                '[', '').replace(
+                ']', '').replace(
+                '_', ' ').replace(
+                '-', ' ').capitalize()
+            if self.code in DIMENSIONS:
+                dimension = DIMENSIONS[code]
+                name = dimension['name']
+            else:
+                try:
+                    additional_dimensions_settings = settings.PHYSICS_ADDITIONAL_DIMENSIONS
+                except AttributeError:
+                    additional_dimensions_settings = ADDITIONAL_DIMENSIONS
+                if code in additional_dimensions_settings.keys():
+                    name = additional_dimensions_settings[code]['name']
+                else:
+                    try:
+                        cd = CustomDimension.objects.get(code=code)
+                        name = cd.name
+                    except CustomDimension.DoesNotExist:
+                        pass
+            self._name = name
+            return name
 
     def __repr__(self):
         """
@@ -388,6 +530,13 @@ class Dimension:
         """
         return self.code
 
+    @property
+    def pint_dimension(self):
+        """
+        Return the Pint dimension object
+        """
+        return self.unit_system.ureg._dimensions.get(self.code)
+
     def _prefixed_units(self, unit_names):
         """
         Add prefixed units to list of units
@@ -406,7 +555,12 @@ class Dimension:
                         self.unit_system.unit(unit_name=prefix + unit))
         return unit_list
 
-    def units(self, user=None, key=None) -> [Unit]:
+    @property
+    def dimensionality(self):
+        return self.unit_system.ureg.get_dimensionality(self.code)
+
+    @property
+    def units(self) -> [Unit]:
         """
         List of units for this dimension
         :param user: optional user for custom units
@@ -415,48 +569,37 @@ class Dimension:
         if self.code == '[compounded]':
             return self._compounded_units
         if self.code == '[custom]':
-            return self._custom_units(user=user, key=key)
-        unit_list = []
-        try:
-            unit_list.append(
-                self.unit_system.unit(
-                    UNIT_SYSTEM_BASE_AND_DERIVED_UNITS[
-                        self.unit_system.system_name][self.code]
+            return self._custom_units(
+                user=self.unit_system.user,
+                key=self.unit_system.key)
+        unit_list = \
+            self.unit_system.ureg._cache.dimensional_equivalents.get(
+                self.dimensionality
+            ) or []
+        unit_names = []
+        for u in unit_list:
+            try:
+                unit_names.append(
+                    Unit(unit_system=self.unit_system, code=u)
                 )
-            )
-        except (KeyError, UnitNotFound):
-            logging.warning(f"unable to find base unit for"
-                            f"unit system {self.unit_system.system_name}"
-                            f" and dimension {self.code}")
-        try:
-            unit_list.extend(
-                [
-                    Unit(unit_system=self.unit_system, pint_unit=unit)
-                    for unit in
-                    self.unit_system.ureg.get_compatible_units(self.code)
-                ])
-        except KeyError:
-            logging.warning(f"Cannot find compatible units "
-                            f"for this dimension {self.code}")
-        unit_names = [str(u) for u in unit_list]
+            except UnitNotFound:
+                logging.info(f"Unit {u} not found "
+                             f"on unit system "
+                             f"{self.unit_system.system_name}")
+                continue
         unit_names.extend(self._prefixed_units(unit_names))
-        return set(sorted(unit_list, key=lambda x: x.name))
+        return sorted(list(set(unit_names)), key=lambda x: x.name)
 
     @property
     def _compounded_units(self):
         """
         List units that do not belong to a dimension
         """
-        available_units = self.unit_system.available_unit_names()
-        dimensioned_units = []
-        for dimension_code in [d for d in DIMENSIONS.keys() if
-                               d != '[compounded]' and d != '[custom]']:
-            dimension = Dimension(
-                unit_system=self.unit_system,
-                code=dimension_code)
-            dimensioned_units.extend([u.code for u in dimension.units()])
-        return [self.unit_system.unit(au)
-                for au in set(available_units) - set(dimensioned_units)]
+        compounded_units = []
+        for unit in self.unit_system.available_units().values():
+            if '[compounded]' in [d.code for d in unit.dimensions]:
+                compounded_units.append(unit)
+        return compounded_units
 
     def _custom_units(self, user: User, key: str = None) -> [Unit]:
         """
@@ -477,17 +620,17 @@ class Dimension:
             return []
 
     @property
-    def base_unit(self):
+    def base_unit(self) -> Unit:
         """
         Base unit for this dimension in this Unit System
         """
         try:
-            return UNIT_SYSTEM_BASE_AND_DERIVED_UNITS[
-                self.unit_system.system_name][self.code]
+            return self.unit_system.unit(UNIT_SYSTEM_BASE_AND_DERIVED_UNITS[
+                                             self.unit_system.system_name][self.code])
         except KeyError:
-            logging.warning(
-                f'dimension {self.dimension} is not part of '
-                f'unit system {self.unit_system.system_name}')
+            logging.info(
+                f'No base unit for dimension {self.code} '
+                f'in unit system {self.unit_system.system_name}')
             return None
 
 
@@ -498,12 +641,14 @@ class Unit:
     unit_system = None
     code = None
     unit = None
+    dimensions_cache = None
 
     def __init__(
             self,
             unit_system: UnitSystem,
             code: str = '',
-            pint_unit: pint.Unit = None):
+            pint_unit: pint.Unit = None
+    ):
         """
         Initialize a Unit in a UnitSystem
         :param unit_system: UnitSystem instance
@@ -517,8 +662,9 @@ class Unit:
             self.code = code
             try:
                 self.unit = getattr(unit_system.system, code)
+                self.dimensions_cache = self.dimensions
             except pint.errors.UndefinedUnitError:
-                raise UnitNotFound("invalid unit for system")
+                raise UnitNotFound(f"invalid unit {code} for system")
         else:
             raise UnitNotFound("invalid unit for system")
 
@@ -553,16 +699,29 @@ class Unit:
         """
         return self.unit_symbol(self.code)
 
+    @property
+    def dimension_codes(self) -> [str]:
+        """
+        Return dimension codes for unit
+        """
+        return [d.code for d in self.dimensions]
+
     @property
     def dimensions(self) -> [Dimension]:
         """
         Return Dimensions of Unit
         """
-        dimensions = [
-            Dimension(unit_system=self.unit_system, code=code) for code in
-            DIMENSIONS.keys()
-            if DIMENSIONS[code]['dimension'] == str(self.dimensionality)]
-        return dimensions or '[compounded]'
+        if self.dimensions_cache:
+            return self.dimensions_cache
+        dimensions = []
+        for d in self.unit_system.available_dimensions().values():
+            if d.dimensionality == self.dimensionality:
+                dimensions.append(d)
+        if not dimensions:
+            return [Dimension(unit_system=self.unit_system,
+                              code='[compounded]'), ]
+        else:
+            return dimensions
 
     @staticmethod
     def base_unit(unit_str: str) -> (str, str):
@@ -635,7 +794,7 @@ class Unit:
         Return dimensionality of a unit in Pint universe
         """
         try:
-            return self.unit_system.ureg.get_base_units(self.code)[1]
+            return self.unit.dimensionality
         except KeyError:
             return ''
 
@@ -820,6 +979,102 @@ class UnitConversionPayload:
         self.eob = eob
 
 
+class CustomDimension(models.Model):
+    """
+    Additional dimension for a user
+    """
+    AVAILABLE_SYSTEMS = (
+        ('Planck', 'Planck'),
+        ('SI', 'SI'),
+        ('US', 'US'),
+        ('atomic', 'atomic'),
+        ('cgs', 'CGS'),
+        ('imperial', 'imperial'),
+        ('mks', 'mks'),
+    )
+    user = models.ForeignKey(
+        User,
+        related_name='dimensions',
+        on_delete=models.PROTECT)
+    key = models.CharField(
+        "Categorization field (e.g.: customer ID)",
+        max_length=255, default=None, db_index=True, null=True, blank=True)
+    unit_system = models.CharField(
+        "Unit system to register the unit in", max_length=20,
+        choices=AVAILABLE_SYSTEMS)
+    code = models.CharField(
+        "technical name of the dimension (e.g.: [myDimension])",
+        max_length=255)
+    name = models.CharField(
+        "Human readable name (e.g.: My dimension)",
+        max_length=255)
+    relation = models.CharField(
+        "Relation to existing dimensions (e.g.: [mass]*[length]/[time])",
+        max_length=255)
+
+    class Meta:
+        """
+        Meta
+        """
+        unique_together = ('user', 'key', 'code')
+        ordering = ['name', 'code']
+
+    def validate_dimensions(self) -> [bool, str]:
+        """
+        Validate dimensions of the relation
+        """
+        dims = {}
+        us = UnitSystem(system_name=self.unit_system,
+                        user=self.user,
+                        key=self.key)
+        for dim_name in re.findall('(?P<dim>\[\w+\])', self.relation):
+            try:
+                dim = Dimension(unit_system=us, code=dim_name)
+                dunits = dim.units
+                if dunits:
+                    dims[dim_name] = f"(1 * {dunits[0].code})"
+            except DimensionNotFound:
+                return False, f"Dimension not found {dim_name}"
+        rel = self.relation
+        for key, value in dims.items():
+            rel = rel.replace(key, value)
+        try:
+            us.ureg.Quantity(rel)
+        except pint.errors.UndefinedUnitError as e:
+            return False, f"Incoherent units {str(e)}"
+        return True, ""
+
+    def save(self, *args, **kwargs):
+        """
+        Save custom unit to database
+        """
+        us = UnitSystem(
+            system_name=self.unit_system,
+            user=self.user,
+            key=self.key
+        )
+        self.code = self.code.replace('-', '_')
+        if self.code[0] != '[':
+            self.code = '[' + self.code
+        if self.code[-1] != ']':
+            self.code = self.code + ']'
+        if self.relation in us.available_dimension_names():
+            raise DimensionDuplicateError("relation already exist")
+        if self.code in us.available_dimension_names():
+            raise DimensionDuplicateError("Dimension code already exists")
+        try:
+            us.add_dimension(
+                code=self.code,
+                relation=self.relation
+            )
+        except ValueError as e:
+            raise DimensionValueError(str(e)) from e
+        is_valid, error = self.validate_dimensions()
+        if not is_valid:
+            raise DimensionDimensionError(error)
+        return super().save(*args, **kwargs)
+
+
 class CustomUnit(models.Model):
     """
     Additional unit for a user
@@ -869,12 +1124,14 @@ class CustomUnit(models.Model):
         """
         us = UnitSystem(system_name=self.unit_system)
         self.code = self.code.replace('-', '_')
-        self.symbol = self.symbol.replace('-', '_')
-        self.alias = self.alias.replace('-', '_')
+        if self.symbol:
+            self.symbol = self.symbol.replace('-', '_')
+        if self.alias:
+            self.alias = self.alias.replace('-', '_')
         if self.code in us.available_unit_names():
             raise UnitDuplicateError
         try:
-            us.add_definition(
+            us.add_unit(
                 code=self.code,
                 relation=self.relation,
                 symbol=self.symbol,
@@ -883,9 +1140,9 @@ class CustomUnit(models.Model):
             raise UnitValueError(str(e)) from e
         try:
             us.unit(self.code).unit.dimensionality
-        except pint.errors.UndefinedUnitError:
-            raise UnitDimensionError
-        return super(CustomUnit, self).save(*args, **kwargs)
+        except (pint.errors.UndefinedUnitError, AttributeError) as e:
+            raise UnitDimensionError(str(e))
+        return super().save(*args, **kwargs)
 
 
 def unit_system_obj(self, *args, **kwargs):
@@ -897,4 +1154,4 @@ def unit_system_obj(self, *args, **kwargs):
 
 
 # Add unit_system_obj attribute to Country class
-setattr(Country, 'unit_system_obj', unit_system_obj)
\ No newline at end of file
+setattr(Country, 'unit_system_obj', unit_system_obj)
