@@ -1,6 +1,11 @@
 """
 Settings for Units module
 """
+from django.utils.translation import gettext_lazy as _
+DOMAINS = {}
+
+# This will display these units in the list of units
+ADDITIONAL_DIMENSIONS = {}
 
 # This will display these units in the list of units
 ADDITIONAL_UNITS = {}
